/*
 * This file is part of exc.
 * Copyright (C) 2012  Guillaume Quintin.
 *
 * cocoju is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <stdio.h>
#include <string.h>

#define VERSION "exc version 0.2"

FILE *in;

int pbc;

int fgetc2(FILE *f) {
  int ret;
  if ( pbc != EOF ) {
    ret = pbc;
    pbc = EOF;
  }
  else {
    ret = fgetc(f);
  }
  return ret;
}

int ungetc2(int c) {
  if ( pbc == EOF ) {
    pbc = c;
    return c;
  }
  return EOF;
}

void skip_string(void) {
  int c;
  for(;;) {
    c = fgetc2(in);
    if ( c == EOF || c == '\"' ) return;
    if ( c == '\\' ) (void)fgetc(in);
  }
}

void skip_char(void) {
  int c;
  for(;;) {
    c = fgetc2(in);
    if ( c == EOF || c == '\'' ) return;
    if ( c == '\\' ) (void)fgetc(in);
  }
}

void skip_c99_comment(void) {
  int c;
  for(;;) {
    c = fgetc2(in);
    if ( c == EOF || c == '\n' ) return;
  }
}

void skip_c89_comment(void) {
  int c;
  for(;;) {
    c = fgetc2(in);
    if ( c == EOF ) return;
    if ( c == '*' ) {
      c = fgetc2(in);
      if ( c == EOF || c == '/' ) return;
    }
  }
}

void exc(void) {
  int c;
  for(;;) {
    c = fgetc2(in);
    if ( c == EOF ) return;
    if ( c == '*' ) {
      c = fgetc2(in);
      if ( c != '/' ) {
        fputc('*',stdout);
        ungetc2(c);
        continue;
      }
      return;
    }
    fputc(c,stdout);
  }
}

void extract(void) {
  int c;
  pbc = EOF;
  for(;;) {
    c = fgetc2(in);
    switch(c) {
      case EOF:
        return;
      case '\"':
        skip_string();
        continue;
      case '\'':
        skip_char();
        continue;
      case '/': {
        int d;
        d = fgetc2(in);
        if ( d == '/' ) {
          skip_c99_comment();
          continue;
        }
        if ( d != '*' ) {
          ungetc2(d);
          continue;
        }
        d = fgetc2(in);
        if ( d != '*' ) {
          ungetc2(d);
          skip_c89_comment();
          continue;
        }
        exc();
        continue;
      }
    }
  }
}

int main(int argc,char **argv) {
  int i;
  FILE *f;
  if ( argc == 2 && !strcmp(argv[1],"-v") ) {
    puts(VERSION);
    return 0;
  }
  if ( argc > 1 ) {
    for( i = 1 ; i < argc ; i++ ) {
      f = fopen(argv[i],"r");
      if ( !f ) {
        fprintf(stderr,"exc: %s: cannot open file\n",argv[i]);
        fflush(stderr);
        return -1;
      }
      if ( f ) {
        in = f;
        extract();
      }
      fclose(f);
    }
  }
  else {
    in = stdin;
    extract();
    fclose(in);
  }
  fflush(stdout);
  return 0;
}

